<?php

namespace WpBeacon\Actions;

use stdClass;
use WpBeacon\Data\PluginInfo;
use WpBeacon\Data\VersionInfo;

class GetPluginVersionsAction
{
    private static string $checkedAt;

    private static array $plugins;

    public static function all(): array
    {
        $pluginInfos = [];

        $updates = get_site_transient('update_plugins');

        self::$checkedAt = date(DATE_ATOM);
        self::$plugins = array_merge_recursive(get_plugins(), get_mu_plugins());

        if (empty($updates) || empty(self::$plugins)) {
            return $pluginInfos;
        }

        return self::mapUpdateTransientOnPluginInfos($updates);
    }

    private static function mapUpdateTransientOnPluginInfos(stdClass $transient): array
    {
        return [
            ...self::mapPluginsOnPluginInfos($transient->response ?? []),
            ...self::mapPluginsOnPluginInfos($transient->no_update ?? []),
        ];
    }
    private static function mapPluginsOnPluginInfos(array $plugins): array
    {
        if ($plugins === []) {
            return $plugins;
        }

        $pluginInfos = [];
        foreach ($plugins as $key => $update) {
            $pluginInfos[] = self::mapUpdatePluginsTransientItemOnPluginInfo($key, $update);
        }

        return $pluginInfos;
    }

    private static function mapUpdatePluginsTransientItemOnPluginInfo(string $key, stdClass $transientItem): PluginInfo
    {
        $currentPlugin = self::$plugins[$key] ?? [];

        $pluginInfo = new PluginInfo(
            $currentPlugin['Name'] ?? $transientItem->slug,
            $transientItem->slug,
            $transientItem->url,
            new VersionInfo(
                $currentPlugin['Version'],
                $transientItem->new_version,
                self::$checkedAt,
                version_compare($currentPlugin['Version'], $transientItem->new_version, 'eq'),
                self::getReleaseDateFromUpdateTransient($transientItem)
            )
        );

        return $pluginInfo;
    }

    private static function getReleaseDateFromUpdateTransient(stdClass $transient): ?string
    {
        if (property_exists($transient, 'release_date')) {
            return  date_create_from_format('Ymd', $transient->release_date)->format(DATE_ATOM);
        }

        return null;
    }
}
